program dnv3e
## performs damped least squares inversion for matrix from lincomb
# pstomo by G. Nolet
#c 1994 Suzan vd Lee
include workdefs
include worklims
      
integer jy(nrfpr),jn(nrfpr)
integer iw(nonzr)
real rw(nonzr)
real u(nsum), v(nminv), w(nminv)
real x(nminv)
real cft(nminv), cfi(nminv), cs(nminv)
equivalence(cfi,cs)
#real rf(nsum)
real apu(nrfpr)
real tarray(2)
character*50 matrix,rhs
character*56 sofm
character*80 bgmodel
logical kscale,twod

common/nrs/nif,depsilon
      
10 format(a,$ )
write(6,*) "program dnv3e:"

write(6,10) 'Give name of matrix: '
read(5,'(a)') matrix
write(6,*) matrix
write(6,10) 'give name of right hand side vector q:'
read(5,'(a)') rhs
write(6,*) rhs

write(6,10) "give damp factor:"
read(5,*) depsilon
if (depsilon > 0.) {
   write(6,*) " damp factor = ",depsilon
} else {
   write(6,*) " damping off" 
}

# Column scaling is applied in two ways: extern and intern. External scaling
# is meant to non-dimensionalize the inverse problem by scaling each type of 
# variable by the size of the average expected anomaly for that type of variable.
# The non-dimensional solution vector is the one that is damped. The ratio of
# the scaling factors for each type can emphasize one variable over the other
# during the inversion.
# Internal scaling does not affect the final solution but optimizes the search
# directions of LSQR. Internal scaling can be turned on/off and can be varied
# through scepsilon.
# At this stage (after hsmcomb, before dnv3e) the external scaling (just as the
# internal scaling) has not been applied yet and can also be adjusted (based on
# output from hsmcomb) in the file columnsums0.

open(9,file='columnfactors0',status='old')
rewind 9
read(9,*) ncolc0, nsq, njy, (apu(j),j=1,njy), jntr, apu5, colmaxapu, scepsilon
close(9)
write(6,*) "done with columnfactors0"
nptot = njy*nsq+jntr
inquire(file="profile.info",exist=twod)
if (twod) {
   open(9,file="profile.info",status="old")
   rewind 9
   read(9,'(a80)') bgmodel
   read(9,*) x1,x2,x3,x4,x5,x6,i7,i8,x9,nsrc,apu6
   close(9)
   write(6,*) "done with profile.info"
} else {
   nsrc = 0
}
nptot = nptot + nsrc
if (nptot != ncolc0) {
   write(6,*) 'ERROR in column counting for ext. scaling'
   write(6,*) nptot, ncolc0
   stop
}

top = 1. + scepsilon
xepsilon = colmaxapu*scepsilon

write(6,10) "column scaling (with quasi-inverse of columnsums) on (T) or off (F)?:"
read(5,*) kscale

# Internal column scaling:
if (kscale) {
   write(6,*) ' int. column scaling on'
   write(6,*) ' column scaling damp-factor = ',scepsilon
   open(1,file='columnsums',status='old')
   rewind 1
   read(1,*) ncolc
   if (ncolc != ncolc0) {
      write(6,*) 'ERROR: incompatible # of columns in ext. and int. scaling'
      stop
   }
   read(1,*) (cs(i),i=1,ncolc)
   close(1)
   write(6,*) 'file columnsums has been read'
   do j=1,njy {
      do i = (j-1)*nsq + 1, j*nsq {
         bot = apu(j)*cs(i) + xepsilon
         cfi(i) = top/bot
      }
   }
   do i = njy*nsq + 1, njy*nsq + jntr {
      bot = apu5*cs(i) + xepsilon
      cfi(i) = top/bot
   }
   do i = njy*nsq + jntr + 1, njy*nsq + jntr + nsrc {
      bot = apu6*cs(i) + xepsilon
      cfi(i) = top/bot
   }
} else {
   write(6,*) " int. column scaling off"
   do i=1,ncolc0 {
      cfi(i) = 1.
   }
}
# External column scaling:
do j=1,njy {
   do i = (j-1)*nsq + 1, j*nsq {
      cft(i) = cfi(i)*apu(j)
   }
}
do i = njy*nsq + 1, njy*nsq + jntr {
   cft(i) = cfi(i)*apu5
}
# added Nov 1 1997:
do i = njy*nsq + jntr + 1, njy*nsq + jntr + nsrc {
   cft(i) = cfi(i)*apu6
}

sofm = 'sizeof'//matrix
write(6,*) sofm
open(2,file=sofm,status='old')
rewind 2
  read(2,*) mrow,ncol,nmmx,nif
close(2)

if (mrow > maxic) stop 'increase maxic (# of rows) in worklims'
if (mrow == 0) stop 'mrow is null'
if (ncol > nminv) stop 'increase nminv (# of columns) in worklims'
if (ncol == 0) stop 'mrow is null'
if (nmmx > nonzr) stop 'increase nonzr (# of non-zeroes) in workdefs'
if (kscale & ncolc != ncol) stop 'columnfactors not right nr of columns'

m = mrow + ncol
n = ncol
lenrw = nmmx
leniw = nmmx

# unfortunately row scaling should happen before column scaling.
# Say, in hsmcomb. Back to 1 and depsilon for now.
# do k = 1, mrow {
#    rf(k) = 1.
# }
# do k = mrow+1, m {
#    rf(k) = depsilon
# }

write(6,*) 'read:'
write(6,*) 'nr of rows:',m
write(6,*) 'nr of real rows:',mrow
write(6,*) 'nr of colums:',n
write(6,*) 'max nr of non-zeroes per row:',leniw
write(6,*) 'number of paths involved:',nif

# get right hand side:
open(2,file=rhs)
rewind 2
do k=1,mrow {
   read(2,*,end=100) u(k) 
}
close(2)
do k=mrow+1,m {
   u(k) = 0. 
}
write(6,*) "give max nr of iterations:"
read(5,*) itmax
write(6,*) itmax
if (itmax < 1 | itmax > 1000) {
   write(6,*) "ridiculous number"
   stop
}
cpu = etime(tarray)
write(6,*) "Entering pstomd..."

call pstomd(m,n,x,u,v,w,itmax,leniw,lenrw,iw,rw,depsilon,cft,cfi,matrix)

cpu = etime(tarray) - cpu
write(6,*) ''
write(6,*) "pstomd took ",cpu," s = ",cpu/60," minutes"

do i = 1,n {
   x(i) = cft(i)*x(i)
}
write(6,*) 'solution rescaled with internal and external columnfactors'
write(6,*) 'solution is thus no longer dimensionless'

#smn = 999
#smx = -999.
#sav = 0.
#do i=1,n {
#   sav = sav + x(i)
#   smx = max(smx,x(i))
#   smn = min(smn,x(i)) }
#sav = sav/n
#write(6,*) 'min, avg and max value of solution: ',smn,sav,smx
do j=1,njy {
   smn = 999
   smx = -999.
   sav = 0.
   do i = (j-1)*nsq + 1, j*nsq {
      sav = sav + x(i)
      smx = max(smx,x(i))
      smn = min(smn,x(i)) 
   }
#   sav = sav/n
   sav = sav/nsq
   write(6,*) 'min, avg and max of apu ',apu(j),' solution: ',smn,sav,smx
}
if (jntr > 0) {
   smn = 999
   smx = -999.
   sav = 0.
   do i = njy*nsq + 1, njy*nsq + jntr {
      sav = sav + x(i)
      smx = max(smx,x(i))
      smn = min(smn,x(i)) 
   }
#   sav = sav/n
   sav = sav/jntr
   write(6,*) 'min, avg and max of apu ',apu5,' solution: ',smn,sav,smx
}

# apu now included in columnfactors, scwapu no longer necessary
# call scwapu(n,x)

open(2,file='solution')
rewind 2
   write(2,*) n
   write(2,*) (x(i), i=1,n)
close(2)

write(6,*) "solution written to file 'solution'"

write(6,*) ''
write(6,*) 'dnv3e finished'
stop 
100 write(6,*) 'premature end to file ',rhs
    write(6,*) 'k= ',k,' m= ',m
end
