#define MAXP     24              # model parameters
define MAXP     50              # model parameters
define MAXTYP	 5		# parameter types (Vp,Vs,Rho,q,h)

program mknwpr

# For version with lots of debug output see ~/tmp/newton5/src

character*12 floldpar
character*16 fcon
character*80 fmod,file1l,file2l,file3l,file1r,file2r,file3r
character*80 backeig, adminfile, rhspredicted
character*5 runnr, chtotnpar
character*3 number,chnpar
character*7 ctext 
#dimension nprr(100000), index(MAXP)
dimension index(MAXP)
dimension eigvect(MAXP,MAXP),stdev(MAXP),qpar(MAXP),qparrhs(MAXP)
dimension eigval(MAXP),par(MAXP),parrhs(MAXP)
dimension r(300), vp(300), vs(300), rho(300), q(300), p(300)
dimension r0(300), alpha(300), beta(300), density(300), quality(300)
dimension itype(MAXP),ip1(MAXP),ip2(MAXP),ip3(MAXP),c(MAXP,MAXTYP)
dimension h(300,MAXP),g(300,MAXTYP,MAXP),gtilde(300,MAXTYP,MAXP),gg(MAXP,MAXP)
dimension jflag(MAXTYP),cav(MAXTYP),nav(MAXTYP)
dimension emod(300,MAXTYP)
dimension f1start(4), f2start(4), f2step(4), f2stop(4), f2min(4), penlim(4)
logical exoldp,exeig,exadm,exrhs
equivalence (emod(1,1),vp),(emod(1,2),vs),(emod(1,3),rho),(emod(1,4),q)
data nav/MAXTYP*0/

data jflag/MAXTYP*0/

narg = iargc()
if (narg != 6) {
  print *,'Correct call is:  mknwpr path_ident prev_#_constraints npar adminfile rhs_predicted background_eigf_file'
  stop 'Fatal error'
}
call getarg(1,runnr)
call getarg(2,chtotnpar)
call getarg(3,chnpar)
call getarg(4,adminfile)
call getarg(5,rhspredicted)
call getarg(6,backeig)

inquire(file=adminfile,exist=exadm)
if (! exadm) {
   write(6,*) 'ERROR: File ', adminfile, ' not found'
   stop
}
inquire(file=rhspredicted,exist=exrhs)
if (! exrhs) {
   write(6,*) 'ERROR: File ', rhspredicted, ' not found'
   stop
}
inquire(file=backeig,exist=exeig)
if (! exeig) {
   write(6,*) 'ERROR: File ', backeig, ' not found'
   stop
}

read(chtotnpar,*) npartotrhs 
read(chnpar,*) nparrhs 

open(2,file='mknwpr.out',status='unknown',access='append')
write(2,*) 'mknwpr: ', runnr, number, nparrhs, adminfile, rhspredicted
write(2,*) runnr,nif

floldpar='oldpar.'//runnr
inquire(file=floldpar,exist=exoldp)
if(exoldp) {				# read results of newton5 run
  print *,'input from ',floldpar
  open(15,file=floldpar,status='old',iostat=ios)
  read(15,*) loopiter
  read(15,*) pen, epsilon
  read(15,*) f1start
  read(15,*) f2start
  read(15,*) f2step
  read(15,*) f2stop
  read(15,*) f2min
  read(15,*) penlim
  read(15,*) epsrel
  read(15,*) maxit
  read(15,*) maxdat
  read(15,*) jdebug
  read(15,*) jsenv
  read(15,*) jsnorm
  read(15,*) jswgt, envlim
  read(15,*) leb
  read(15,*) jhess
  read(15,*) findif
  read(15,*) itmax
  read(15,fmt='(a72)') file1l
  read(15,fmt='(a72)') file2l
  read(15,fmt='(a72)') file3l
  read(15,fmt='(a72)') file1r
  read(15,fmt='(a72)') file2r
  read(15,fmt='(a72)') file3r
  read(15,*) npar
  do i=1,npar
    read(15,*) par(i),jspar
  do i=1,npar {
    read(15,*) i,qpar(i),stdev(i),eigval(i)
    read(15,*) (eigvect(j,i),j=1,npar)	# (oldpar writes V in columns)
  }
  do i=1,npar
    read(15,*) i,itype(i),ip1(i),ip2(i),ip3(i),(c(i,j),j=1,MAXTYP)

  read(15,*) evla,evlo,stla,stlo
  close(15)
  print *,' '
  print *,'Succesful read of oldpar file: ',floldpar
} else {
  print *,'File ',floldpar,' does not exist on this directory'
  stop 'Fatal error for param5'
}

# decouple:

do i=1,MAXTYP {
   cav(i) = 0.
   nav(i) = 0
}

moho=0						# moho layer, if any
do i=1,npar {
  it=itype(i)
  jflag(it)=1
  if(it==5) moho=ip1(i)
  ct=c(i,it)
  nav(it)=nav(it)+1
  cav(it)=cav(it)+ct
  do j=1,MAXTYP
    c(i,j)=0.
  c(i,it)=ct
}
do it=1,MAXTYP {
  if(nav(it)>0) cav(it)=cav(it)/nav(it)		# average weights
}

# find model limits
ipmin=ip1(1)
ipmax=ip3(1)
do i=2,npar {
  ipmin=min(ipmin,ip1(i))
  ipmax=max(ipmax,ip3(i))
}

# read model

fmod=file2l
if (fmod(1:12)=='Love2-undef') fmod=file2r
inquire(file=fmod,exist=exeig)
ios=-1
if(exeig)open(11,file=fmod,status='old',form='unformatted',iostat=ios)
if(ios<0) {
  print *,' '
  print *,'File open error on ',fmod
  stop 'Fatal error'
}


read(11) radius,in,nr,nfreq, nbran,df,cmax
if(n.gt.300) stop 'model has too many layers, increase 300'
do i=1,nr
  read(11) r(i),vp(i),vs(i),rho(i),q(i)
close(11)
zmoho=0.
if(moho>0) zmoho=6371000.-r(moho)	# save moho depth (m)

# throw away unused model layers
nr=ipmax-ipmin+1
do i=ipmin,ipmax {
  r(i-ipmin+1)=r(i)
  vp(i-ipmin+1)=vp(i)
  vs(i-ipmin+1)=vs(i)
  rho(i-ipmin+1)=rho(i)
  q(i-ipmin+1)=q(i)
}
rmin = r(1)

rscale=r(nr)-r(1)  # scale length for depth, doesn't really matter.

do i=1,npar {
  ip1(i)=ip1(i)-ipmin+1
  if(ip2(i)>0)ip2(i)=ip2(i)-ipmin+1
  if(ip3(i)>0)ip3(i)=ip3(i)-ipmin+1
}

# construct h(r)
call makeh(npar,h,r,nr,ip1,ip2,ip3,itype)

# construct g(r)
call makeg(npar,itype,h,eigvect,cav,c,g,nr)

# construct gtilde
call makegg(g,npar,r,nr,rscale,gg)
call invertgg(gg,npar)
call makegtilde(gg,g,npar,nr,gtilde)

# now scale gtilde to full constraint kernel G:
do i=1,npar {
  do j=1,4 {
    if(cav(j)==0.) next
    do k=1,nr {
      if(emod(k,j)==0.) next
      gtilde(k,j,i)=gtilde(k,j,i)/(rscale*cav(j)*emod(k,j))
    }
  }
}
# here we change from r(Moho) to z(Moho)=6371000.-r(Moho) and therefore
# switch sign of gtilde(1,5,i):
if(cav(5)!=0.)
  do i=1,npar
    gtilde(1,5,i)=-gtilde(1,5,i)/cav(5)

# scale to unit variance
do k=1,npar
  do j=1,MAXTYP
    do i=1,nr
      gtilde(i,j,k)=gtilde(i,j,k)/stdev(k)


#nparrhs=12
do i=1,nparrhs {
   qparrhs(i) = 0.
}
# determine nprr(i) (number of non-dropped constraints) 
# for i=1,nif (# of constraint files)
# and for i=nif the indexes of the non-dropped constraints.
open(10,file=adminfile)
rewind 10

# read previous constraints:
nprrtot = npartotrhs
print*, nprrtot
do i=1,npartotrhs {
    read(10,*) anum,ctext
    if (ctext=="dropped") {
       nprrtot = nprrtot - 1
    }
}
print*, nprrtot

do j=1,nparrhs {
   index(j) = j
}
nprr = nparrhs
print*, nparrhs
do j=1,nparrhs {
   read(10,*) anum,ctext
   if (ctext=="dropped") {
      nprr = nprr - 1
      na = nint(anum)
      index(na) = 0
   }
}
close(10)
nprrt = nparrhs
i=1
do while (i<=nprr) {
   if (index(i)==0) {
      nprrt = nprrt - 1
      do j=i,nprrt {
	 index(j) = index(j+1)
      }
   } else {
      i = i+1
   }
}
print*,(index(j),j=1,nprr)

open(9,file=rhspredicted,status='old')
rewind 9

# read previous q from rhs:
do i = 1,nprrtot {
   read(9,*) dum
}
do i=1,nprr {
   read(9,*) qparrhs(index(i))
}
close(9)

do i=1,nparrhs {
   print*, qparrhs(i)
}

# subtract background model
do i=1,npar {
  x=0.
  do j=1,4 {
    if(jflag(j)==0) next
    do k=1,nr 
      p(k)=gtilde(k,j,i)*emod(k,j)
    call trap(p,r,1,nr,sum)
    x=x+sum
  }
  x=x+gtilde(1,5,i)*zmoho	# Mohochange (zmoho>0 but gtilde switched sign)
  qparrhs(i)=qparrhs(i)-x
write(2,*) 'sum',x,'to subtract'
}

open(8,file=backeig,form='unformatted')
rewind 8
read(8) radius,in,nr0,nfreq,nbran,df,cmax
do i=1,nr0 {
   read(8) r0(i),vp(i),vs(i),rho(i),q(i)
}
# Attention: (Moho depth of hsmcomb background model)
#zmh35 = 35000.
# changed (FM) 17.06.2003
zmh35 = 40000.
# changed (SL) 12.11.2001
#zmh35 = 30000.

do i=1,nr {write(3,*) (6371000.-r(i))/1000.,gtilde(i,2,1)}
call convm(nr,r,nr0,r0,vp,vs,rho,q,gtilde,MAXTYP,npar,alpha,beta,density,quality)
do i=1,nr {write(4,*) (6371000.-r(i))/1000.,gtilde(i,2,1)}
call addbk(npar,jflag,nr,r,gtilde,alpha,beta,density,quality,zmh35,qparrhs)

do i=1,nparrhs {
   qparrhs(i) = qparrhs(i)*stdev(i)
}
do i=1,nparrhs {
   print*, qparrhs(i),stdev(i)
}

do i=1,npar {
   parrhs(i) = 0.
   do j=1,nparrhs   {
      parrhs(i) = parrhs(i) + eigvect(i,j)*qparrhs(j)
   }
}
npar=nparrhs

# write  newpar
fcon='newpar.'//runnr
open(7,file=fcon)
rewind 7
write(7,*) -9
write(7,*) pen, epsilon
write(7,*) f1start
write(7,*) f2start
write(7,*) f2step
write(7,*) f2stop
write(7,*) f2min
write(7,*) penlim
write(7,*) epsrel
write(7,*) 0
write(7,*) maxdat
write(7,*) jdebug
write(7,*) jsenv
write(7,*) jsnorm
write(7,*) jswgt, envlim
write(7,*) leb
write(7,*) jhess
write(7,*) findif
write(7,*) itmax
write(7,fmt='(a72)') file1l
write(7,fmt='(a72)') file2l
write(7,fmt='(a72)') file3l
write(7,fmt='(a72)') file1r
write(7,fmt='(a72)') file2r
write(7,fmt='(a72)') file3r
write(7,*) npar
do i=1,npar {
   write(7,*) parrhs(i),0
}
close(7)

end

#--------------------------------------------------------

subroutine makeh(npar,h,r,nr,ip1,ip2,ip3,itype)

# expands h(r) definition to levels r

# this assumes all parameters are uncoupled, i.e. parameter i
# is connected to only one physical variable (Vp,Vs,Rho,q or one
# discontinuity change.

# h(i,j) contains the value of  the physical parameter (e.g. Vs)
# at radius r(i), multiplied with the standard deviation for that
# parameter as given in c(j,itype(j)). If itype(j)=5 (discontinuity)
# h(1,j) has the st dev for the discontinuity level. In the present
# version, only one discontinuity change is allowed (generally the Moho).

dimension itype(MAXP),ip1(MAXP),ip2(MAXP),ip3(MAXP)
dimension h(300,MAXP),r(300)

# initialize to 0
do i=1,npar 
  do j=1,nr
    h(j,i)=0.

ndh=0
do i=1,npar {
  it=itype(i)
  if(it==5) {		# special treatment for h
    ndh=ndh+1
    if(ndh>1) stop 'Newton5 can handle only one discontinuity change'
    h(1,i)=1	 	# use only the first element of h!
    next
  }
  if(ip2(i)==0) {	# boxcar h(r)
    do j=ip1(i),ip3(i)
      h(j,i)=1
    next
  }
  r1=r(ip1(i)); r2=r(ip2(i)); r3=r(ip3(i))	# triangle h
  d12=r2-r1; d23=r3-r2
  if(d12>0.) {
    do j=ip1(i)+1,ip2(i)
      h(j,i)=(r(j)-r1)/d12
  }
  if(d23>0.) {
    do j=ip2(i),ip3(i)-1
      h(j,i)=(r3-r(j))/d23

  }
}

return
end

#----------------------------------------------------------------

subroutine makeg(npar,itype,h,eigvect,cav,c,g,nr)   

dimension h(300,MAXP),g(300,MAXTYP,MAXP),cav(MAXTYP),c(MAXP,MAXTYP)
dimension eigvect(MAXP,MAXP),itype(MAXP)

# constructs g(j)=sum s(i,j)*h(i)
# the h(i) belong to one parameter type only, but the crosscorrelation
# among parameters makes that g(j) is multidimensional

# initialize
do k=1,npar
  do j=1,MAXTYP
    do i=1,nr
      g(i,j,k)=0.

do j=1,npar {
  do i=1,npar {
    it=itype(i)
    if(cav(it)>0.)
      delta=c(i,it)/cav(it)
    else
      delta=0.
    if(it==5) {			#	again special treatment for Moho
      g(1,5,j)=g(1,5,j)+eigvect(i,j)*h(1,i)*delta
      next
    }
    do k=1,nr
      g(k,it,j)=g(k,it,j)+eigvect(i,j)*h(k,i)*delta
  }
}

return
end

#----------------------------------------------------------------

subroutine makegg(g,npar,r,nr,rscale,gg)

dimension g(300,MAXTYP,MAXP),r(300),gg(MAXP,MAXP),p(300)

# constructs a matrix of which element Gij is the inner product
# between g(i) and g(j)

do j=1,npar
  do i=1,npar
    gg(i,j)=0.

do i=1,npar {
  do j=i,npar {
    gg(i,j)=0.
    do it=1,4 {
      do k=1,nr 
	p(k)=g(k,it,j)*g(k,it,i)
      call trap(p,r,1,nr,sum)		# inner product Vs,Vp,Rho,q
      gg(i,j)=sum/rscale+gg(i,j)
    }
    gg(i,j)=gg(i,j)+g(1,5,i)*g(1,5,j)	# inner product for Moho
    gg(j,i)=gg(i,j)
  }
}


return
end

#----------------------------------------------------

subroutine trap(y,x,m,n,sum)
real x(n),y(n),sum

sum=0.0
do i=m,n-1 {
  sum = sum + 0.5*(y(i)+y(i+1))*abs(x(i+1)-x(i)) 
}

return
end

#-----------------------------------------------------------------------------


subroutine invertgg(gg,npar)

dimension gg(MAXP,MAXP), v(MAXP,MAXP), rhs(MAXP), xxx(MAXP), q(MAXP)
#dimension gdebug(MAXP,MAXP)
data rhs/MAXP*0./,xxx/MAXP*0./

# use minfit to find inverse of gg (rhs and xxx are used as dummies)
call minfit(gg,MAXP,rhs,xxx,v,npar,npar,q,0.)


do i=1,npar {
  do j=1,npar {
    sum=0.
    do k=1,npar
      sum=sum+v(i,k)*v(j,k)/q(k)
    gg(i,j)=sum; gg(j,i)=sum
  }
}

return
end

#--------------------------------------------------------------------


subroutine makegtilde(gg,g,npar,nr,gtilde)

dimension g(300,MAXTYP,MAXP),gtilde(300,MAXTYP,MAXP),gg(MAXP,MAXP)

do k=1,npar {
  do it=1,5 {
    do i=1,nr {
      sum=0.
      do j=1,npar
	sum=sum+gg(k,j)*g(i,it,j)
      gtilde(i,it,k)=sum
    }
  }
}

return
end

