subroutine pstomd(m,n,x,u,v,w,itmax,leniw,lenrw,iw,rw,depsilon,cft,cfi,matrix)
define nitw 14
character*80 tomoint
integer iw(leniw)
real rw(lenrw)
dimension x(n),u(m),v(n),w(n)
dimension xprev(13e5),dx(13e5),dxprev(13e5)
dimension itw(nitw)
real cfi(n), cft(n)
#real rf(m)
character*50 matrix
data itw/1,2,3,4,5,7,9,12,15,18,21,25,30,40/

#   subroutine to solve the linear tomographic problem Ax=u using the 
#   lsqr algorithm (C.C.Paige and M.A.Saunders, ACM Trans.Math.Softw. 
#   8, 43 - 71 and 195 - 209, 1982. 
#
#   Input: m is the number of data, n the number of unknowns, u contains
#      the data (is overwritten), itmax is the number of iterations.
#   Output: x is the solution
#   Scratch: arrays v(n) and w(n)
#   Subroutines: routines avpu and atupv must be supplied by the user.
#      avpu(m,n,u,v) computes u=u+A*v for given input u,v 
#      atupv(m,n,u,v) computes v=v+A(transpose)*u for given u,v 

20 format(a4,1x,a10,1x,a10,1x,a2,1x,a10,1x,a7,1x,a7,1x,a10,1x,a10,1x,a1)
30 format(i4,1x,f10.4,1x,f10.7,1x,a2,1x,f10.4,1x,f7.3,1x,f7.3,1x,f10.4,1x,f10.4,3x,f5.3)
write(6,*) 'give path/filename for enormous tomo.int:'
read(5,'(a)') tomoint
write(6,*) ' writing to:',tomoint
write(6,*) ''
open (9,file=tomoint,form='unformatted')
write(9) n
dxnorm = 0.
xnorm = 0.
do i=1,n { 
   dx(i)=0
   x(i)=0
   v(i)=0 
}			# initialize
call normlz(m,u,beta) 
b1=beta 
call dpror(2,m,n,v,u,leniw,lenrw,iw,rw,cft,cfi,matrix)
call normlz(n,v,alfa)
rhobar=alfa 
phibar=beta 
do i=1,n { w(i)=v(i) }
write(6,20) 'iter ','phibar ','r ','--','||dx|| ','cosdd ','cosd ','||Ax-b|| ','||x|| ','r'
write(6,30) 0,beta,1.,'--',0.,0.,0.,beta, 0.,1.
do iter =1,itmax {				# repeat 
   a= - alfa 
   do i=1,m { u(i)=a*u(i) }		# bidiagonalization
   call dpror(1,m,n,v,u,leniw,lenrw,iw,rw,cft,cfi,matrix)
   call normlz(m,u,beta) 
   b= - beta 
   do i=1,n { v(i)=b*v(i) }
   call dpror(2,m,n,v,u,leniw,lenrw,iw,rw,cft,cfi,matrix)
   call normlz(n,v,alfa)
   rho=sqrt(rhobar*rhobar+beta*beta)		# modified QR 
   c=rhobar/rho 
   s=beta/rho 
   teta=s*alfa 
   rhobar= - c*alfa 
   phi=c*phibar 
   phibar=s*phibar 
   t1=phi/rho 
   t2= - teta/rho
   dxprevnorm = dxnorm
   xprevnorm = xnorm
   do i=1,n { 
      x(i)=t1*w(i)+x(i) 
      w(i)=t2*w(i)+v(i) 
   }	# update 
   r=phibar/b1 
   call getnorm(n,x,cfi,xnormsq)
   xnorm = sqrt(xnormsq)
   axminbsq = phibar*phibar - depsilon*depsilon*xnormsq
   sqaxminbsq = sqrt(axminbsq)
   ddxinp = 0.
   dxinp = 0.
   do i=1,n { 
      realx = cfi(i)*x(i)
      dx(i) = realx - xprev(i)
      ddxinp = ddxinp + dx(i)*dxprev(i)
      dxinp = dxinp + dx(i)*xprev(i)
      xprev(i) = realx
      dxprev(i) = dx(i)
   }
   call getdnorm(n,dx,addxnormsq)
   dxnorm = sqrt(addxnormsq)
   if (dxprevnorm > 0.) {
      ddxinp = ddxinp/dxprevnorm
   } else {
      ddxinp = 0 }
   if (xprevnorm > 0.) {
      dxinp = dxinp/xprevnorm
   } else {
      dxinp = 0 }
   if (dxnorm > 0.) {
      ddxinp = ddxinp/dxnorm
      dxinp = dxinp/dxnorm
   } else  {
      ddxinp = 0
      dxinp = 0 }
   write(6,30) iter,phibar,r,'--',dxnorm,ddxinp,dxinp,sqaxminbsq,xnorm,sqaxminbsq/b1
   do i = 1,nitw {
      if (iter == itw(i)) {
#         write(9) iter,x,v,alfa,beta,phibar,r		# intermediate output
#        intermediate output, with x corrrected for int. scaling:
         write(9) iter,(xprev(j),j=1,n),v,alfa,beta,phibar,r 
      }
   }
   }
return 
end					# return


subroutine normlz(n,x,s)			# normalizes vector x 
dimension x(n)
s=0. 
do i=1,n { s=s+x(i)**2 }
s=sqrt(s) 
ss=1./s 
do i=1,n { x(i)=x(i)*ss }
return 
end

subroutine getnorm(n,x,c,s)
dimension x(n),c(n)
s=0. 
do i=1,n { s=s+(c(i)*x(i))**2 }
return 
end

subroutine getdnorm(n,x,s)
dimension x(n)
s=0. 
do i=1,n { s=s+x(i)*x(i) }
return 
end
