subroutine quadrx(y,x,m,n,sum)

# quadrature of y at supports x. some x may be equal
# but array x should otherwise be monotonic
# should be exact for quadratic curves as long as all
# subintervals have at least 3 x supports.
# quadrature interval is x(m) to x(n)

implicit double precision (a-h,o-z)
real x(n),y(n),sum
data hlf,thrd/0.5d0,0.33333333333333d0/

if(n<=m) {sum=0.; return }

if(n==m+1) { sum=0.5*(y(n)+y(m))*(x(n)-x(m)); return }

sumq=0.d0
ii=m
eps=1.0d-6*abs(x(n)-x(m))/abs(n-m)

while(ii<n) {

  ii=ii+1
  x1=x(ii-1); x2=x(ii); y1=y(ii-1); y2=y(ii); d12=x1-x2

  if(abs(d12)<eps) { 					# discontinuity
    sumq=sumq-hlf*((y1+y2)*d12)
    next 
  }	

  ii=ii+1

  if(ii>n) {						# past end of x
    ii=ii-1; x3=x2; x2=x1; x1=x(ii-2)			# move down 1
    y3=y2; y2=y1; y1=y(ii-2)
    d12=x1-x2; d13=x1-x3; d23=x2-x3
    if(abs(d12)<eps | abs(d23)<eps) {
      sumq=sumq-hlf*(y2+y3)*d23
      break
    }
    a=y1/(d12*d13); b=-y2/(d12*d23); c=y3/(d13*d23)	
    at=a+b+c; bt=-a*(x2+x3)-b*(x1+x3)-c*(x1+x2)
    ct=a*x2*x3+b*x1*x3+c*x1*x2				
    sumq=sumq+((thrd*at*x3+hlf*bt)*x3+ct)*x3-((thrd*at*x2+hlf*bt)*x2+ct)*x2 
    break
  }

  x3=x(ii); y3=y(ii); d23=x2-x3; d13=x1-x3

  if(abs(d23)<eps) {					# discontinuity
    sumq=sumq-hlf*((y1+y2)*d12+(y2+y3)*d23)
    next
  }

  if(abs(d12-d23)<eps) {				# equal interval
    sumq=sumq-d13*(y1+4.0d0*y2+y3)/6.0d0		# Simpson saves time
    next
  }
  
  a=y1/(d12*d13); b=-y2/(d12*d23); c=y3/(d13*d23)	# Lagrange interpol
  at=a+b+c; bt=-a*(x2+x3)-b*(x1+x3)-c*(x1+x2)
  ct=a*x2*x3+b*x1*x3+c*x1*x2				# parabolic coefficients

  sumq=sumq+((thrd*at*x3+hlf*bt)*x3+ct)*x3-((thrd*at*x1+hlf*bt)*x1+ct)*x1 

}

sum=sumq						# back to single prec


return
end
