/* 
 * tri.c
 * Spherical triangular tesselation codes.
 * ANSI version adapted from the previous standard C.
 * Zheng Wang 11/16/94
 */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fcntl.h>
#include "tri.h"

double **STT_alloc(int nt)
{
        int nrh,nch,nrl=0,ncl=0,i;
        double **m;
      assert(nt);
       nrh=3*nt;
 
        m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
        if (!m) nrerror("allocation failure 1 in STT_alloc()");
        m -= nrl;
 
        for(i=0;i<=nrh;i++) {
               nch=5*i;
              if(i>2*nt)
                nch=5*(3*nt-i);
                else if(i>=nt)
                 nch=5*nt;
               if(i==0 || i==3*nt) nch=1; 
                m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
                if (!m[i]) nrerror("allocation failure 2 in STT_alloc()");
                m[i] -= ncl;
        }
        return m;
}


void STT_free(double **m,int nt)
{
        int i,nrh,nrl=0,ncl=0;
        
	assert(nt);
        nrh=3*nt;
        for(i=nrh;i>=nrl;i--)
           free((char*) (m[i]+ncl));
          free((char*) (m+nrl));
}

void ReArrange(int nt, double **ta, double **pa)
{
   int i,j,jplus;
   double plusj;
   double *ttmp, *ptmp;

   ttmp=(double *)malloc((unsigned) (3*nt)*sizeof(double));
   ptmp=(double *)malloc((unsigned) (5*nt)*sizeof(double));
   assert(ttmp);
   assert(ptmp);

/* 
now shift the arrays so that they start at edges of icosahedron triangles
instead of at the minimum non-negative longitude 
*/
   plusj = 0.1;
   jplus = (int)(plusj);
   for (i=nt+1;i<2*nt;i++) {
       plusj = plusj + 0.5;
       jplus = (int)(plusj);
       for (j=0;j<jplus;j++) {
           ttmp[j] = ta[i][j];
           ptmp[j] = pa[i][j];
       }
       for (j=0;j<5*nt-jplus;j++) {
           ta[i][j] = ta[i][j+jplus];
           pa[i][j] = pa[i][j+jplus];
       }
       for (j=0;j<jplus;j++) {
           ta[i][5*nt-1-jplus+j] = ttmp[j];
           pa[i][5*nt-1-jplus+j] = ptmp[j];
       }

       for (j=0;j<jplus;j++) {
           ttmp[j] = ta[4*nt-i][j];
           ptmp[j] = pa[4*nt-i][j];
       }
       for (j=0;j<5*(i-nt)-jplus;j++) {
           ta[4*nt-i][j] = ta[4*nt-i][j+jplus];
           pa[4*nt-i][j] = pa[4*nt-i][j+jplus];
       }
       for (j=0;j<jplus;j++) {
           ta[4*nt-i][5*(i-nt)-1-jplus+j] = ttmp[j];
           pa[4*nt-i][5*(i-nt)-1-jplus+j] = ptmp[j];
       }
   }
   i = 2*nt;
   plusj = plusj + 0.5;
   jplus = (int)(plusj);
   for (j=0;j<jplus;j++) {
       ttmp[j] = ta[i][j];
       ptmp[j] = pa[i][j];
   }
   for (j=0;j<5*nt-jplus;j++) {
       ta[i][j] = ta[i][j+jplus];
       pa[i][j] = pa[i][j+jplus];
   }
   for (j=0;j<jplus;j++) {
       ta[i][5*nt-1-jplus+j] = ttmp[j];
       pa[i][5*nt-1-jplus+j] = ptmp[j];
   }
    
/*
   for (i=0;i<=3*nt;i++) {
       printf(" %d,%f,%f\n",i,ta[i][0],pa[i][0]); 
   }
*/

   printf("ReArrange done\n"); 
   free(ttmp);
   free(ptmp);
}
 
/*#################################################################*/

void tri_(float *ddd,float *dmx,float *col,int *nt1,
	 int *imx1, int *ntr,float tt[mxtr],float pp[mxtr])
{
	 int n,nt,l0,i,j,i1,j1,k1,k;
	 int nn,imx;
	 long nv;
	 double dmin,dmax,tmin;
         double ddel,**ta,**pa;
	 double *icot,*icop;

nt = (int)((ICO*radius)/(*ddd)+0.5);
*nt1 = nt;
fprintf(stdout,"nt = %d\n",*nt1);
ta=STT_alloc(nt);
pa=STT_alloc(nt);

/* 
* For n-sided triangular polyhedron, the number of the vertices are:
* nv=n/2+2;
*/
      nv=10*nt*nt+2;
      fprintf(stdout,"number of vertices for whole sphere would be: %d\n",nv);

	   ddel=side(nt);
	   dmax = sidex(nt);
	   *dmx = dmax;
	   dmin = siden(nt);
	   fprintf(stdout,"minimum triangle-side length, %f, km\n",radius*dmin);
	   fprintf(stdout,"average triangle-side length, %f, km\n",radius*ddel);
	   fprintf(stdout,"maximum triangle-side length, %f, km\n",radius*dmax);

	  AssmlTria(nt,ta,pa);
	  ReArrange(nt,ta,pa);

	    /*
            for(k=0;k<=3*nt;k++){
              i1=5*k-1;
                if(k>=nt)i1=5*nt-1;
                if(k>2*nt)i1=5*(3*nt-k)-1;
                  if(i1<0)i1=0;
                  for(j=0;j<=i1;j++){
	             printf("%f %f \n",ta[k][j]/rad,pa[k][j]/rad); 
                  }
             }
	     */


   tmin = 0.;
   i = 0;
   while ((float)tmin < (*col) & i < 3*nt) {
       i = i+1;
       tmin = PI;
       i1=5*i;
       if (i>=nt) i1 = 5*nt;
       if (i>2*nt) i1 = 5*(3*nt-i);
       for (j=0;j<i1;j++) {
           if (ta[i][j] != 0.) {
              tmin = min(tmin,ta[i][j]);
           }
       }    
   }
   imx = i;
   *imx1 = imx;
   printf("need %d lat-circles (excl. pole) to cover whole area\n",imx);

/*
count the vertices with k. index for tt,pp is then k-1.
also ta is colatitude tt = pi/2 - ta.
*/
 
   nn=0;
   k = 1; 
   tt[k-1] = PI/2.-(float)ta[0][0]; 
   pp[k-1] = (float)pa[0][0];
   for (i=1;i<=imx;i++) {
       if (i <= nt) {
           for (j=0;j<5*i;j++) {
               k = 1 + 5*i*(i-1)/2 + j+1;
               tt[k-1] = PI/2.-(float)ta[i][j];
               pp[k-1] = (float)pa[i][j]; }
       } else if (i <= 2*nt) {
           for (j=0;j<5*nt;j++) {
               k = 1 + 5*nt*(nt-1)/2 + (i-nt)*5*nt + j+1;
               tt[k-1] = PI/2.-(float)ta[i][j]; 
               pp[k-1] = (float)pa[i][j]; }
       } else if (i < 3*nt) {
           for (j=0;j<5*(3*nt-i);j++) {
               k = 1 + 5*nt*(nt-1)/2 + 5*nt*nt + 5*(3*nt*i - i*(i-1)/2 - 4*nt*nt - nt) + j+1;
               tt[k-1] = PI/2.-(float)ta[i][j]; 
               pp[k-1] = (float)pa[i][j]; }
       } else if (i == 3*nt) {
           k = 1 + 10*nt*nt + 1;
           tt[k-1] = PI/2.-(float)ta[3*nt][0]; 
           pp[k-1] = (float)pa[3*nt][0];
       } else if (i == 3*nt+1) {
           fprintf(stderr,"Warning: imx > 3nt, imx = %d, 3nt = %d\n",imx,3*nt);
           fprintf(stderr,"i.e. the whole earth is included\n");
       }
   }
   nn = k;
   *ntr = nn;
 
   for (k=0;k<nn;k++) {
/*      printf(" %f %f ", tt[k],pp[k]);  */
       if (tt[k] == 0. && pp[k] == 0.) {
          fprintf(stderr,"\nerror: tt,pp NOT properly filled for k= %d\n\n",k);
   }}

/* return EXIT_SUCCESS; */
}

/*#############################################################*/

double side(int nf)
{
      int i,k;
      double dc,tn,t1,s[20],a[20];

      dc=ICO/nf;
      tn=0;
      for(i=0;i<nf;i++){
      t1=sin(36./RAD)*sin(dc*(double)(i+1));
      t1=2*asin(t1)/(double)(i+1);
      tn+=t1;
      }
return (tn/nf);
}

double sidex(int nf)
{
      int i,k;
      double dc,tn,t1,s[20],a[20];
      double tmx;

      tmx=0;
      dc=ICO/nf;
      tn=0;
      for(i=0;i<nf;i++){
      t1=sin(36./RAD)*sin(dc*(double)(i+1));
      t1=2*asin(t1)/(double)(i+1);
      tmx = max(t1,tmx);
      }
return (tmx);
}

double siden(int nf)
{
      int i,k;
      double dc,tn,t1,s[20],a[20];
      double tmn;

      tmn = 40000;
      dc=ICO/nf;
      tn=0;
      for(i=0;i<nf;i++){
      t1=sin(36./RAD)*sin(dc*(double)(i+1));
      t1=2*asin(t1)/(double)(i+1);
      tmn = min(t1,tmn);
      }
return (tmn);
}

void AssmlTria(int nt,double **ta,double **pa)
{
	 int i,j,k,k1,n,n0,n2;
	 double *icot,*icop;
	 double *ta0,*pa0;

  
  icot=dvector(0,nt*nt);
  icop=dvector(0,nt*nt);
  ta0=dvector(0,5*nt);
  pa0=dvector(0,5*nt);

    

 for(i=1;i<=5;i++)
    {
	 TriaTessel0(nt,nt,icot,icop);
	 n=0;
	 for(k=0;k<=nt;k++){
	   for(j=0;j<=k;n++,j++){
	      n0=(i-1)*k+j;
	      if(j<k || !k){
	     ta[k][n0]=*(icot+n);
	     pa[k][n0]=(i-1.)*72./RAD+ *(icop+n);
	     n2=n0+k/2;
	     if(n2>=5*k) n2=n2-5*k;
	     ta[3*nt-k][n2]=PI-*(icot+n);
	     pa[3*nt-k][n2]=(i-.5)*72./RAD+ *(icop+n);
	     if(pa[3*nt-k][n2]>PI2) 
	          pa[3*nt-k][n2]-=PI2;
	     if((PI2-pa[3*nt-k][n2])<1.e-4) 
	         pa[3*nt-k][n2]=0.;
	      }
	     }
	     }
     }

 for(i=6;i<=15;i++)
    {
	 TriaTessel1(nt,i,icot,icop);
	 n=0;
	 for(k=1;k<nt;k++){
	    if(i%2)k1=k;
	      else k1=nt-k;
	   for(j=0;j<=k1;n++,j++){
	      if(i%2) n0=((i-6)/2)*nt+nt-k+j;
		else n0=((i-6)/2)*nt+j;
	     if(j<k1){
	     ta[nt+k][n0]=*(icot+n);
	     pa[nt+k][n0]=*(icop+n);
	      }
	     }
	    }
     }

     for(k=nt+2;k<=2*nt-1;k++){
      for(j=0;j<5*nt;j++){
	pa0[j]=pa[k][j];
	ta0[j]=ta[k][j];
	}
	n=(k-nt)/2;
      for(j=0;j<5*nt;j++){
        if((j+n)<5*nt){
          pa[k][j+n]=*(pa0+j);
          ta[k][j+n]=*(ta0+j);
	  }
	  else{
          pa[k][j+n-5*nt]=*(pa0+j);
          ta[k][j+n-5*nt]=*(ta0+j);
	  }

	  }
    }	    
    ta[0][0]=pa[0][0]=pa[3*nt][0]=0.;
    ta[3*nt][0]=PI;

free_dvector(icot,0,nt*nt);
free_dvector(icop,0,nt*nt);
free_dvector(ta0,0,5*nt);
free_dvector(pa0,0,5*nt);
 }

void TriaTessel0(int nt,int np,double *icot,double *icop)
{
      int n,i,k;
      double t0=0.,p0=0.,t1,p1,t2,p2,phi=72.,c0=63.435;
      double dt,dp;
      float ft0,ft1,fp0,fp1,zs,det,al,bt,ga;

      c0/=RAD; phi/=RAD;
      dt=c0/nt;
      n=0; 
      det=al=bt=ga=0.;
      dp=0.;
      for(i=0;i<=nt;i++){
	if(i)dp=phi/(i);
        t0=t1=i*dt;p0=0.;p1=phi;
	ft0=ft1=t0;fp0=p0;fp1=p1;
        if(i)
	  euler(ft0,fp0,ft1,fp1,&det,&al,&bt,&ga); 
	 for(k=0;k<=i;n++, k++){
	   if(k==0 || k==i)
	      {
               *(icot+n)=i*dt;*(icop+n)=k*dp; 
	       }
	     else{
               ft1=PI/2.;fp1=k*det/(i);         
               rot(-ga,-bt,-al,&ft1,&fp1,&zs);
	       t1=ft1;p1=fp1;
	       if(PI2-p1<1.e-4)p1=0.;
               *(icot+n)=t1;*(icop+n)=p1; 
	         }
	    }
	 }
}


void TriaTessel1(int nt,int np,double *icot,double *icop)
{
      int n,i1,i,k,nn;
      double t0=0.,p0=0.,t1,p1,t2,p2,phi=72.,c0=63.435;
      double b0,dg,dt,dt1,dp1,dp;
      float ft0,ft1,fp0,fp1,zs,det,al,bt,ga;

      c0/=RAD; phi/=RAD;
      dt=ICO/(double)(nt);
      n=0; 
      det=al=bt=ga=0.;
      dp=0.;
      nn=((np-5)/2);

      for(i=1;i<nt;i++){
/*
*  using the formula on p52 of Mathematical Handbook (Chinese) 
*  to find the longitude at given point on the spherical triangles of
*  icosahedrons.
*/
	 if(i>(nt/2)){
	 dp1=sin((double)phi/2.)*sin((double)(nt-i)*dt);
	 dp1=asin((double)dp1);
	 dt1=cos((double)(nt-i)*dt)/cos((double)dp1);
	 dt1=acos((double)dt1);
         t0=t1=PI-c0-dt1;
	 b0=c0+dt1;
	}
	 else{
	 dp1=sin((double)phi/2.)*sin((double)i*dt);
	 dp1=asin((double)dp1);
	 dt1=cos((double)i*dt)/cos((double)dp1);
	 dt1=acos((double)dt1);
        t0=t1=c0+dt1;
	 b0=c0+dt1;
	}
	 dg=cos(dp1)*cos(b0);
	 dg=acos(dg);
	 dp=sin((double)(b0))/tan((double)(dp1));
	 dp=1./dp;
	 dp=atan((double)(dp));
	 if(i>(nt/2))dp=phi/2.-dp;

	if(np%2){
	   p0=(double)(nn)*phi-dp;
	   p1=(double)(nn)*phi+dp;
	   i1=i;
	   }
	   else{
	   p0=(double)(nn)*phi+dp;
	   p1=(double)(nn+1.)*phi-dp;
	   i1=nt-i;
	   }
	ft0=ft1=t0;fp0=p0;fp1=p1;
	  euler(ft0,fp0,ft1,fp1,&det,&al,&bt,&ga); 

	 for(k=0;k<=i1;n++, k++){
               ft1=PI/2.;fp1=k*det/i1;
               rot(-ga,-bt,-al,&ft1,&fp1,&zs);
	t1=ft1;p1=fp1;
	       if(PI2-p1<1.e-4)p1=0.;
               *(icot+n)=t1;*(icop+n)=p1; 
	    }
	 }
}
