/* 
 * tri.h 
 */

#define radius   6371.0              /* radius of the earth          */
#define rad      0.017453292519943   /* (pi/180.0) */
#define mxtr   610000    

/* useful constants */

#define PI 3.1415926535897932
#define PI2 6.28318530717958
#define RAD (180./PI)
#define ICO (63.435/RAD)
#define RA 6371000. /* radius of the Earth */
#define GRAV 6.6723e-11 /* gravitational constant */
#define RHO 5515.0 /* average density of the Earth */
#define OMS (sqrt(PI*GRAV*RHO))
#define VMD  (RA*RA*RHO*RHO*PI*GRAV)
#define VSC  (RA*OMS)

/* useful functions */


#define SIGN(i) ((i<0) ? -1:1)
#define EVEN(i) ((i-2*((i)/2)==0) ? 1:0)
#define ODD(i) ((i-2*((i)/2)==0) ? 0:1)
#define MOD(i,j) (i-((i)/(j))*(j))
#define MODF(a,b) (a-(int)((a)/(b))*(b))

/* 
 * Define struct for spherical triangular tesselation
 */


typedef struct STT{
     int nt;    /* Tesselation frequency */
     float th0; /* Lower bound for colatitude */
     float th1; /* upper bound for colatitude */
     float ph0; /* lower bound for longitude */
     float ph1; /* upper bound for longitude */
     } STT;


/* 
 * Declarations of interfaces
 */
void    STT_free(double **m, int nt);
double **STT_alloc(int nt);
/*
 * allocate and free allocated space given prefered tesselation frequency nt
 * STT_alloc returns a matrix.
 */
void tri_(float *ddd,float *dav,float *col,int *nt1,
	  int *imx, int *ntr, float *tt,float *pp); 
double side(int nf);
double sidex(int nf);
double siden(int nf);
void Search(int nt,double th,double ph,double **ta,double **pa,int *i,int *j);
void AssmlTria(int nt,double **ta,double **pa);
void ReArrange(int nt, double **ta, double **pa);
void TriaTessel0(int nt,int np,double *icot,double *icop);
void TriaTessel1(int nt,int np,double *icot,double *icop);
float *vector(int nl,int nh);
double *dvector(int nl,int nh);
float **matrix(int nrl,int nrh,int ncl,int nch);
double **dmatrix(int nrl,int nrh,int ncl,int nch);
void free_vector(float *v,int nl,int nh);
void free_dvector(double *v,int nl,int nh);
void free_matrix(float **m,int nrl,int nrh,int ncl,int nch);
void free_dmatrix(double **m,int nrl,int nrh,int ncl,int nch);
void nrerror(char *error_text);
void rot(float alpha,float beta,float gamma,float *theta,float *phi,float *zeta);
void reduce(float *pth,float *pph);
void euler(float sco,float slo,float rco,float rlo,float *delta,
float *alpha,float *beta,float *gamma);
/*
 * STT_convert extracts the grids in the region defined by structure STT 
 * and converts a global 2D matrix to a vector. It returns the total number
 * fallen in theat region.
 */

#define max(a,b) ((a>b) ? a : b)
#define min(a,b) ((a<b) ? a : b)
